/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "dfbpoint.h"

#include "action.h"
#include "context.h"
#include "header.h"
#include "image.h"
#include "listitem.h"
#include "parser.h"
#include "render.h"
#include "resources.h"
#include "slide.h"
#include "slides.h"
#include "slides-view.h"
#include "text.h"
#include "video.h"


#define DFBCHECK(x...)                                                     \
               err = x;                                                    \
               if (err != DFB_OK) {                                        \
                    fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ ); \
                    DirectFBErrorFatal( #x, err );                         \
               }


static DFBResult  err;


IDirectFB *dfb           = NULL;
gchar     *document_root = NULL; 


typedef GType (* GetTypeFunc) (void);

static GetTypeFunc item_get_type_funcs[] =
{
  action_get_type,
  header_get_type,
  image_get_type,
  listitem_get_type,
  slide_get_type,
  slides_get_type,
  video_get_type,
  NULL
};

static void
items_init (void)
{
  GetTypeFunc *funcs;
  GType        type;

  for (funcs = item_get_type_funcs; *funcs; funcs++)
    {
      type = (*funcs) ();
      g_type_class_ref (type);
    }
}

static void
show_slides (Slides *slides)
{
  IDirectFBInputDevice  *keyboard;
  IDirectFBInputBuffer  *keybuffer;
  DFBInputEvent          evt;
  SlidesView *view;
  Slide      *slide;
  GList      *list;       
  gboolean    quit = FALSE;

  DFBCHECK (dfb->GetInputDevice (dfb, DIDID_KEYBOARD, &keyboard));
  DFBCHECK (keyboard->CreateInputBuffer (keyboard, &keybuffer));

  view = slides_view_new (slides);

  while (!quit)
    {
      keybuffer->WaitForEvent (keybuffer);

      while (!quit && keybuffer->GetEvent (keybuffer, &evt) == DFB_OK)
        {
          if (evt.type == DIET_KEYPRESS)
            {
              switch (evt.keycode)
                {
                case DIKC_ESCAPE:
                  quit = TRUE;
                  break;
        
                case DIKC_HOME:
                  slides_view_first_slide (view);
                  break;

                case DIKC_SPACE:
                case DIKC_PAGEDOWN:
                case DIKC_RIGHT:
                  slides_view_next_slide (view);
                 break;

                case DIKC_BACKSPACE:
                case DIKC_PAGEUP:
                case DIKC_LEFT:
                  slides_view_previous_slide (view);
                  break;

                default:
                  slide = slides_view_get_current_slide (view);
                  if (slide)
                    {
                      for (list = slide->items;
                           list;
                           list = g_list_next (list))
                        {
                          Action *action; 
                          
                          if (!IS_ACTION (list->data))
                            continue;
                          
                          action = (Action *) list->data;
                          
                          if (evt.keycode == action->keycode)
                            {
                              action_execute (dfb, action);
                              break;
                            }
                        }
                    }
                  break;
                }
            }
        }
    }

  g_object_unref (view);
}

int
main (int   argc,
      char *argv[])
{
  Slides *slides = NULL;
  gchar  *filename;

  DFBCHECK (DirectFBInit (&argc, &argv));
  g_type_init ();
  items_init ();

  if (argc > 0 && argv[1] && *argv[1])
    {
      if (strcmp (argv[1], "-v") == 0 ||
          strcmp (argv[1], "--version") == 0)
        {
          g_print ("DFBPoint version " VERSION "\n");
          return 0;
        }

      filename = argv[1];
      slides = parse (filename);
    }
  else
    {
      g_print ("DFBPoint version " VERSION
               " - a presentation viewer using DirectFB\n\n");
      g_print ("Usage: %s [options] <filename>\n\n", argv[0]);      
      g_print ("Options:\n"
               "  -v, --version    Print version information.\n");
      return 1;
    }

  if (g_path_is_absolute (filename))
    document_root = g_dirname (g_strdup (filename));
  else
    document_root = g_dirname (g_strconcat (g_get_current_dir (), 
                                            G_DIR_SEPARATOR_S, 
                                            filename, NULL));
  
  if (slides)
    {
      g_print ("got %d slides\n", g_list_length (slides->slides));

      DFBCHECK (DirectFBCreate (&dfb));

      if (item_prepare (ITEM (slides)))
        {
          g_print ("all items prepared\n");

          show_slides (slides);
        }
      else
        {
          g_printerr ("Failed to prepare at least one item "
                      "(see errors above).\n");
        }

      g_object_unref (slides);

      resources_free ();
    }
  else
    {
      g_printerr ("No slides found.\n");
    }

  g_free (document_root);

  return 0;
}
